from views.mainScreen import MainFrame
from models.data import TextData
from Ai.interence import VeraMindInference
from Ai.llm import ArticleRater


class MainFrameController:
    
    def __init__(self,frame:MainFrame) -> None:
        self.frame = frame
        self.model_inference = VeraMindInference('VeraMind-Mini')
        self.rater = ArticleRater()
        
    def get_textdata(self) -> TextData:
        text_data = TextData()
        text_data.url = self.frame.entry_url.get()
        if text_data.text_from_url():
            text_data.text = self.frame.input_textbox.get("0.0", "end")
        
        return text_data
    
    def press_check_button(self):
        text_data = self.get_textdata()
        print(f"text:{text_data}")
        self.prediction(text_data)
        self.frame.output_textbox.configure(state="normal")
        self.frame.output_textbox.delete("0.0", "end")
        
        response_stream = self.rater.get_response(text_data.text, text_data.confidence, text_data.result)
        display_chunks = []
        highlight_chunks = []
        
        DISPLAY_CHUNK_SIZE = 1  # Display as each chunk arrives
        HIGHLIGHT_BATCH_SIZE = 5
        OVERLAP_SIZE = 2

        for chunk in response_stream:
            display_chunks.append(chunk)
            highlight_chunks.append(chunk)

            # Display chunk
            if len(display_chunks) == DISPLAY_CHUNK_SIZE:
                start_index = self.frame.output_textbox.index("end-1c")
                self.frame.output_textbox.insert("end", ''.join(display_chunks))
                self.frame.output_textbox.see("end")  # Scroll to the end
                self.frame.update_idletasks()  # Update the textbox immediately
                display_chunks = []

            # Process highlighting
            if len(highlight_chunks) >= HIGHLIGHT_BATCH_SIZE:
                start_index = self.frame.output_textbox.index(f"end-{sum(len(c) for c in highlight_chunks)}c")
                end_index = self.frame.output_textbox.index("end-1c")
                self.highlight_words(start_index, end_index)

                # Maintain overlap and reset for next batch
                highlight_chunks = highlight_chunks[-OVERLAP_SIZE:]

        # Highlight remaining chunks if any
        if highlight_chunks:
            start_index = self.frame.output_textbox.index(f"end-{sum(len(c) for c in highlight_chunks)}c")
            end_index = self.frame.output_textbox.index("end-1c")
            self.highlight_words(start_index, end_index)

        self.frame.output_textbox.configure(state="disabled")


    def highlight_words(self, start_index, end_index):
        content = self.frame.output_textbox.get(start_index, end_index)
        print(content)
        current_index = start_index

        while True:
            # Find "FAKE"
            fake_index = content.find("FAKE")
            if fake_index != -1:
                fake_pos_start = f"{current_index}+{fake_index}c"
                fake_pos_end = f"{fake_pos_start}+4c"
                self.frame.output_textbox.tag_add("fake_color", fake_pos_start, fake_pos_end)
                self.frame.output_textbox.tag_config("fake_color", foreground="red")
                content = content[fake_index + 4:]
                current_index = fake_pos_end
            else:
                break

        current_index = start_index
        content = self.frame.output_textbox.get(start_index, end_index)

        while True:
            # Find "REAL"
            real_index = content.find("REAL")
            if real_index != -1:
                real_pos_start = f"{current_index}+{real_index}c"
                real_pos_end = f"{real_pos_start}+4c"
                self.frame.output_textbox.tag_add("real_color", real_pos_start, real_pos_end)
                self.frame.output_textbox.tag_config("real_color", foreground="green")
                content = content[real_index + 4:]
                current_index = real_pos_end
            else:
                break

        
    def prediction(self, text_data:TextData) -> TextData:
        result = self.model_inference.predict(text_data.text)
        text_data.confidence = result["confidence"]
        text_data.result = result["result"] 
        print(f"Prediction: {text_data.result}")
        print(f"Confidence: {text_data.confidence}")
        return text_data